/*  Дерево автоматизмов

Это дерево активируется при:
1. Всегда при любых событиях с Пульта – так же как дерево рефлексов, но если к ветке привязан автоматизм, то он выполняется преимущественно, блокируя рефлексы потому, что уже было произвольностью преодолено действие рефлекса при выработке автоматизма. Такой автоматизм обладает меткой успешности ==1. Успешность ==0 означает предположительный вариант действий, а успешность < 0 – заблокированный вариант действия. Так что к ветке может быть прикреплено множество неудачных и предположительных автоматизмов и только один удачный. Более удачный результат переводит ранее удачный автоматизм в предполагаемые.
2. При произвольной активации отдельных условий.
Отсуствие подходящей для данных условий ветки дерева вызывает Ориентировочный рефлекс привлечения внимания к активной ветке с осмыслением ситуации
и рассмотрением альтернатив действиям (4 уровня глубины рассмотрения).
При формировании нового предположительного действия создается новая ветка дерева и к ней прикрепляется автоматизм.
Т.е. новые условия не создают новой ветки, а тольно - новый автоматизм, а пока нет автоматизма будет ориентировочный рефлекс.

У дерева фиксированных 6 уровней.
Для оптимизации поиска по дереву перед узлом Verbal идет узел первого символа : var symbolsArr из word_tree.go
*/

package psychic

import (

)

// инициализирующий блок - в порядке последовательности инициализаций
// из psychic.go
func automatizmTreeInit(){

	loadAutomatizmTree()

}
/////////////////////////////////////////////////////////////

////// ДЕРЕВО автоматизмов имеет фиксированных 6 уровней (кроме базового нулевого)
type AutomatizmNode struct { // узел дерева автоматизмов
	ID int
	BaseID int // базовое состояние
	EmotionID int // стиль поведения - сочетание активностей Базовых контекстов  - ID BaseStyleArr
	ActivityID int // образ сочетания действия с Пульта
	ToneMoodID int  // образ контекста сообщения: сочетание Tone и Mood из структуры vrbal
	SimbolID int
	VerbalID int

	Children []AutomatizmNode // дочерние узлы (ветвление) НЕ АДРЕСА, А РЕАЛЬНЫЕ ОБЪЕКТЫ
	ParentID int     // ID родителя
	ParentNode *AutomatizmNode  // адрес родителя
}
var AutomatizmTree AutomatizmNode
var AutomatizmTreeFromID=make(map[int]*AutomatizmNode)
////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////////
/* попытка активации дерева автоматизмов, если неудачно - начать искать вариант действий
Используется активная текущая информационная среда из psychic.go:
var PsyBaseID=0 // текущее базовое состояние, может быть произвольно изменено
var PsyEmotionImg *Emotion // текущая эмоция Emotion, может быть произвольно изменена
var PsyActionImg *Activity // текущий образ сочетания действий с Пульта Activity
var PsyVerbImg *Verbal // текущий образ фразы с Пульта Verbal
*/
var detectedActiveLastNodID=0
// нераспознанный остаток
var CurrentAutomatizTreeEnd []int
var currentStepCount=0
func automatizmTreeActivation()(int){
	detectedActiveLastNodID=0
	CurrentAutomatizTreeEnd=nil
	currentStepCount=0

	var VerbImgID=0
	var SimbolID=0
	var ToneMoodID=0
	if PsyVerbImg!=nil {
		ToneMoodID = getToneMoodID(PsyVerbImg.ToneID, PsyVerbImg.MoodID)
	}
	// вытащить 3 уровня условий в виде ID их образов
	condArr:=getActiveConditionsArr(PsyBaseID, PsyEmotionImg.ID,PsyActionImg.ID,ToneMoodID,SimbolID,VerbImgID)
	// основа дерева
	cnt := len(AutomatizmTree.Children)
	for n := 0; n < cnt; n++ {
		node := AutomatizmTree.Children[n]
		lev1 := node.BaseID
		if condArr[0] == lev1 {
			detectedActiveLastNodID=node.ID
			ost:=condArr[1:]
			if len(ost)==0{

			}

			conditionAutomatizmFound(1,ost, &node)

			break // другие ветки не смотреть
		}
	}
// Новая ветка создается только если для нее есть автоматизм!

	// результат поиска:
	if detectedActiveLastNodID>0{
// есть ли нейчтенные условия?
		// TODO сформировать CurrentAutomatizTreeEnd и если есть, то - ориентировочный рефлекс 1
// если нет неучтенных условий АВТОМАТИЗМ ДОЛЖЕН БЫТЬ ОБЯЗАТЕЛЬНО потому, что новая ветка создается только если для нее есть автоматизм

// есть ли автоматизм для этой ветки
		automatizmID:=getAutomatizmFromNodeID(detectedActiveLastNodID)
			if automatizmID>0{
			// TODO проверить подходит ли автоматизм к текущим условиям, если нет, - режим нахождения альтернативы  - ориентировочный рефлекс 2

				//нарастить недостающую ветку
// formingBranch(detectedActiveLastNodID, currentStepCount, condArr)
			return automatizmID // блокировка рефлексов
		}

// если нет автоматизма, включить режим поиска
		// TODO  - ориентировочный рефлекс 3
	}else{// вообще нет совпадений для данных условий
		// TODO  - ориентировочный рефлекс 0

// 		formingBranch(AutomatizmTree.ID, currentStepCount, condArr)
	}

	return 0
}
//////////////////////////////////////////////////////////////////

func conditionAutomatizmFound(level int,cond []int,node *AutomatizmNode){
	if cond==nil || len(cond)==0{
		return
	}

	ost:=cond[1:]

	for n := 0; n < len(node.Children); n++ {
		cld := node.Children[n]
		var levID = 0
		switch level {
		case 0:
			levID = cld.BaseID
		case 1:
			levID = cld.EmotionID
		case 2:
			levID = cld.ActivityID
		case 3:
			levID = cld.SimbolID
		case 4:
			levID = cld.ToneMoodID
		case 5:
			levID = cld.VerbalID
		}
		if cond[0] == levID {
			detectedActiveLastNodID=cld.ID
		}

		level++
		currentStepCount=level
		conditionAutomatizmFound(level,ost, &node.Children[n])
		return // раз совпало, то другие ветки не смотреть
	}

	return
}
////////////////////////////////////////////////////////



